classdef CNiHuLab < CGuiModule
    %CNIHULAB Main module of NiHu.Lab
    
    properties (Access = private)
        hGreetingFigure;    % Handle of Greeting Figure
        pBar;               % Handle of Progress Bar in Greeting Figure
        
        Modules         % CUiTabGroup Object containing the Modules
        
        Hardware        % The selected Hardware
        ChSetup         % The Channel Setup Module
        Calibration     % The Calibration Module
        Monitor         % The Monitor Module
        Record          % The Record Module
        Transfer        % The Transfer measurement Module
        DirChar         % The Direction Characteristics measurement Module
        Impact          % The Impact Testing Module
        Distortion      % The Impact Testing Module
        StreamViewer    % The Stream Viewer Module
        
        AcqSetup        % Common AcqSetup Class
        LevelMeter      % Common Level Meter Class
        Excitation      % Common Excitation Class
        Average         % Common Averaging Class
        Trigger         % Common Trigger Class
        DistSignal      % Common Distortion Signal Class
        FileSelect      % Common File Selector Class
        CalSelect       % Common Calibrator Selector Class
    end
    
    methods (Access = private)
        % Initialize the Main Figure of the Software
        function InitFigure(obj)
            % Create the Menu Bar
            m = uimenu('parent', obj.hContainer, ...
                'Label', CBabel.getInstance().getText('SETUP'), ...
                'Tag', 'SETUP');
            am = uimenu('parent', m,...
                'Label', CBabel.getInstance().getText('LANGUAGE'), ...
                'Tag', 'LANGUAGE');
            l = CBabel.getInstance().getLanguages();
            for i = 1 : length(l)
                uimenu('parent', am,...
                    'Label', l{i}, ...
                    'Tag', '', ...
                    'Callback', @(src,event)langCallback(obj,l{i}));
            end
            
            m = uimenu('parent', obj.hContainer, ...
                'Label', '?', ...
                'Tag', '?');
            uimenu('parent', m,...
                'Label', CBabel.getInstance().getText('ABOUT'), ...
                'Tag', 'ABOUT', ...
                'Callback', @(src,event)aboutCallback(obj));
            
            % Arrange the GUI on screen
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
        end
        
        function InitGreeting(obj)
            % The Greeting figure is shown during software initialisation.
            
            % Draw the small figure
            obj.hGreetingFigure = figure('MenuBar', 'none',...
                'NumberTitle', 'off',...
                'Name', 'NiHu.Lab', ...
                'CloseRequestFcn', '', ...
                'Visible', 'off', ...
                'units', 'pixels');
            % Read and draw the NiHu.Lab greeting image
            im = imread(fullfile(fileparts(mfilename('fullpath')), 'data', 'nihulab.png'));
            hFig = axes('parent', obj.hGreetingFigure, ...
                'units', 'pixels',...
                'position', [0 0 size(im,2), size(im,1)]);
            image(im, 'parent', hFig);
            axis(hFig, 'off');
            
            % Create the status bar
            hBar = axes('parent', obj.hGreetingFigure, ...
                'units', 'pixels',...
                'position', [0 0 size(im,2), 10], ...
                'xTick', [], 'yTick', []);
            set(hBar, 'xLim', [0 1], 'yLim', [0 1]);
            obj.pBar = patch([0 0 0 0]', [0 0 1 1]', [.3 .4 .5], ...
                'parent', hBar);
            
            % Arrange Status Bar and Image
            l = {{hFig, [0 0]}; {hBar, [0 0]}};
            GuiLayout.ApplyLayout(obj.hGreetingFigure, l);
            
            % Make figure center and visible
            movegui(obj.hGreetingFigure, 'center');
            set(obj.hGreetingFigure, 'Visible', 'on');
        end
    end
    
    methods
        function obj = CNiHuLab(choices)
            obj = obj@CGuiModule('NiHu.Lab');
            
            % Find a Hardware module
            obj.Hardware = FindHardware(choices);
            
            % Select modules to use
            selection = SelectModules();
            
            % Initialise Greetings Figure
            InitGreeting(obj);
            % Initialisation is done in 14 steps
            n = 18; i = 0;
            % Step 1: Create the UiTabGroup object
            obj.Modules = CUiTabGroup(obj.getContainer());
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 2: Create the ChannelSetup object
            obj.ChSetup = CChannelSetup(obj.Modules, obj.Hardware.InputModules, obj.Hardware.OutputModules);
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 3: Create the AcqSetup object
            obj.AcqSetup = CAcqSetup();
            obj.AcqSetup.FsRange = obj.Hardware.getFsRange();
            obj.AcqSetup.nSamplesRange = 2.^(7:19);
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 4: Create the LevelMeter object
            obj.LevelMeter = CLevelMeter(obj.ChSetup.InputChannels);
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 5: Create the Excitation object
            obj.Excitation = CExcitation();
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 6: Create the Excitation object
            obj.DistSignal = CDistSignal();
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 7: Create the Average object
            obj.Average = CAverage(true);
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 8: Create the Trigger object
            obj.Trigger = CTrigger();
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 9: Create the CalibratorSelect object
            obj.CalSelect = CCalSelect();
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            
            % Step 10: Create the Monitor Module
            if ismember('Monitor', selection)
                obj.Monitor = CMonitor(obj.Modules, obj.Hardware, obj.ChSetup, obj.AcqSetup, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 11: Create the Calibrator Module
            if ismember('Calibration', selection)
                obj.Calibration = CCalibration(obj.Modules, obj.Hardware, obj.ChSetup, obj.AcqSetup, obj.CalSelect, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 12: Create the Stream Viewer Module
            if ismember('Stream Viewer', selection)
                obj.StreamViewer = CStreamViewer(obj.Modules);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 13: Create the Record Module
            if ismember('Record', selection)
                obj.Record = CRecord(obj.Modules, obj.Hardware, obj.ChSetup, obj.AcqSetup, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 14: Create the Transfer Module
            if ismember('Transfer', selection)
                obj.Transfer = CTransfer(obj.Modules, obj.Hardware,...
                    obj.ChSetup, obj.AcqSetup, obj.Excitation,...
                    obj.Average, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 15: Create the Direction Characteristics Module
            if ismember('Direction Characteristics', selection)
                obj.DirChar = CDirChar(obj.Modules, obj.Hardware,...
                    obj.ChSetup, obj.AcqSetup, obj.Excitation,...
                    obj.Average, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 16: Create the Impact Testing Module
            if ismember('Impact Testing', selection)
                obj.Impact = CImpact(obj.Modules, obj.Hardware, obj.ChSetup, obj.AcqSetup, obj.Trigger, obj.Average, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 17: Create the Distortion Measurement Module
            if ismember('Distortion', selection)
                obj.Distortion = CDistortion(obj.Modules, obj.Hardware,...
                    obj.ChSetup, obj.AcqSetup, obj.DistSignal, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            % Step 18: Create the Spectral Monitor Module
            if ismember('Spectral Monitor', selection)
                obj.Impact = CSpectralMonitor(obj.Modules, obj.Hardware, obj.ChSetup, obj.AcqSetup, obj.Trigger, obj.Average, obj.LevelMeter);
            end
            i = i+1; set(obj.pBar, 'xData', (i/n)*[0 1 1 0]'); drawnow;
            
            % Initialize the Main Figure of the Software
            obj.InitFigure();
            
            % Close the Greetings Window
            set(obj.hGreetingFigure, 'CloseRequestFcn', 'closereq');
            close(obj.hGreetingFigure);
        end % of constructor
        
        function langCallback(obj, lang)
            % The Language Change Callback function called by the uimenu
            CBabel.getInstance().setCurrentLanguage(lang);
            CBabel.getInstance().reTranslate(obj.getContainer());
        end
        
        function aboutCallback(~)
            % The About figure is shown during software initialisation.
            
            % Draw the small figure
            fig = figure('MenuBar', 'none',...
                'NumberTitle', 'off',...
                'Name', 'About NiHu.Lab', ...
                'Visible', 'off');
            % Read and draw the NiHu.Lab greeting image
            figures = {
                'DSCF6333.JPG'
                'DSCF8273.JPG'
                'IMG_2825.jpg'
                'IMG_2883.jpg'
                'IMG_2958.jpg'
                };
            idx = ceil(length(figures)*rand());
            im = imread(fullfile(fileparts(mfilename('fullpath')), 'data', figures{idx}));
            hFig = axes('parent', fig, 'position', [0 0 1 1]);
            imagesc(im, 'parent', hFig);
            axis(hFig, 'off');
            
            % Make figure center and visible
            movegui(fig, 'center');
            set(fig, 'Visible', 'on');
        end
        
        function delete(obj)
            disp('NiHuLab destructor called');
        end
    end % of Methods
end
