classdef CSimulator < CNi9234Hardware
    %CSIMULATOR Simulated hardware class
    %   This class implements a hardware simulator
    
    properties (Constant = true)
        DMADepth = 2e6;         % Depth of input and output DMA on HOST side
        TriggerBlockSize = 300; % Block size used at software trigger
    end
    
    properties
        InputModules     % vector of CModule objects
        OutputModules     % vector of CModule objects
        nCh         % number of channels
        
        nPretrigSamples     % number of PreTrigger Samples
        nPosttrigSamples    % number of PostTrigger Samples
        nSamples            % total number of samples to acquire
        
        ImpactBuffer        % internal Buffer for impact testing hw trigger
        ImpactBufferIndex   % index pointer of circular buffer
    end
    
    properties (SetObservable = true)
        nSamplesInBuffer    % number of samples in buffer
    end
    
    methods
        % Constructor
        function obj = CSimulator()
            obj = obj@CNi9234Hardware();
            obj.InputModules = [
                CModule.NI9234a
                CModule.NI9234b
                CModule.NI9215
                ];
            obj.OutputModules = [
                CModule.NI9263a
                ];
        end
        
        function status = MonitorInit(obj, Channels)
            % Initialise Monitor Module
            
            ChSel = cell2mat({Channels.isSelected});
            obj.nCh = length(find(ChSel));
            status = 0;
        end
        
        function [status, starttime] = MonitorStart(obj, ~, ~)
            % Start Monitoring
            
            starttime = now();
            status = 0;
        end
        
        function [status, voltage, elemRemaining] = MonitorAcq(obj, nSamples)
            % Acquire samples during monitoring
            
            voltage = rand(nSamples, obj.nCh)-.5;
            voltage = voltage * diag(1./(1:obj.nCh));
            elemRemaining = 0;
            status = 0;
        end
        
        function [status, rest] = MonitorStop(obj)
            % Stop Monitoring
            
            rest = zeros(0, obj.nCh);
            status = 0;
        end
        
        function status = MonitorClose(obj)
            % Close Monitor Module
            
            status = 0;
        end
        
        function status = TransferInit(obj, InputChannels, ~)
            %TRANSFERINIT Initialise Simulator for Transfer module
            
            ChSel = cell2mat({InputChannels.isSelected});
            obj.nCh = length(find(ChSel));
            status = 0;
        end
        
        function [status, y]= TransferAcq(obj, nPre, nAverage, ~, ~, x)
            %TRANSFERACQ Acquire transfer functions
            
            if size(x,2) > 1
                x = x(:,nPre+(1:nAverage));
            end
            
            y = bsxfun(@times, x, ones(1,1,obj.nCh));
            
            status = 0;
        end
        
        function status = TransferClose(obj)
            % Close Simulator in Transfer Module
            
            status = 0;
        end
        
        function status = ImpactInit(obj, Channels)
            % Initialise Simulator for Impact module
            
            ChSel = cell2mat({Channels.isSelected});
            obj.nCh = length(find(ChSel));
            status = 0;
        end
        
        function status = ImpactArm(obj, nSamples, nPretrigSamples, ~)
            % Arm impact testing (collect pretrigger samples)
            
            obj.nSamples = nSamples;
            obj.nPretrigSamples = nPretrigSamples;
            obj.ImpactBuffer = zeros(obj.nSamples, obj.nCh);
            index = 1:obj.nPretrigSamples;
            obj.ImpactBuffer(index, :) = index(:) * (1:obj.nCh);
            obj.ImpactBufferIndex = obj.nPretrigSamples;
            status = 0;
        end
        
        function [status, isTriggered] = ImpactWaitForTrigger(obj, trigLevel, trigCh)
            % Read a block and search for trigger event inside the block
            
            isTriggered = false;
            index = obj.ImpactBufferIndex+(1:obj.TriggerBlockSize);
            block = index(:) * (1:obj.nCh);
            indTrig = find(abs(block(:, trigCh)) > trigLevel, 1, 'first');
            if ~isempty(indTrig)
                obj.nPosttrigSamples = obj.nSamples - obj.nPretrigSamples - (size(block,1) - indTrig + 1);
                if obj.nPosttrigSamples < 0
                    block = block(1 : end-abs(obj.nPosttrigSamples), :);
                    index = index(1 : end-abs(obj.nPosttrigSamples));
                    obj.nPosttrigSamples = 0;
                end
                isTriggered = true;
            end
            obj.ImpactBuffer(mod(index-1, obj.nSamples)+1, :) = block;
            obj.ImpactBufferIndex = obj.ImpactBufferIndex + size(block,1);
            status = 0;
        end
        
        function [status, y]= ImpactAcq(obj)
            % Read post trigger samples from Simulator and stop acquisition
            
            index = obj.ImpactBufferIndex + (1:obj.nPosttrigSamples);
            obj.ImpactBuffer(mod(index-1, obj.nSamples)+1,:) = index(:) * (1:obj.nCh);
            obj.ImpactBufferIndex = obj.ImpactBufferIndex + obj.nPosttrigSamples;
            obj.ImpactBufferIndex = mod(obj.ImpactBufferIndex-1, obj.nSamples)+1;
            
            y = circshift(obj.ImpactBuffer, -obj.ImpactBufferIndex);
            
            status = 0;
        end
        
        function status = ImpactClose(obj)
            % Close impact testing module
            
            status = 0;
        end
    end % of methods
end % of class CSimulator
