%CSPECETRALMONITOR Nihu.Lab module for Spectral Monitoring with Impact Testing
classdef CSpectralMonitor < CUiTab & CChSetupSensitive
    properties
        Hardware        % handle to CHardware object
        AcqSetup        % handle to CAcqSetup object
        AcqSetupView    % handle to CAcqSetupView object
        Trigger         % handle to CTrigger object
        TriggerView     % handle to CTriggerView object
        Average         % handle to CAverage object
        AverageView     % handle to CAverageView object
        LevelMeter      % handle to CLevelMeter object
        FileSelect      % handle to File Selector object
        FileSelectView  % handle to File Select View object
        
        MeasurementResult   % measurement results are stored in this structure
        
        ThirdTransferData         % CGraphData object (FRF)
        ThirdTransferDataView     % handle to CGraphDataView object
        RespData        % CGraphData object (response time history)
        RespDataView    % handle to CGraphDataView object
        ThirdData         % CGraphData object (FRF)
        ThirdDataView     % handle to CGraphDataView object
        CohData         % CGraphData object (Coherence)
        CohDataView     % handle to CGraphDataView object
    end
    
    methods
        function obj = CSpectralMonitor(ParentGroup, hardware, chsetup, acqsetup,...
                trigger, average, levmeter)
            obj = obj@CUiTab(ParentGroup, 'SPECTRAL_MONITOR_MODULE');
            obj = obj@CChSetupSensitive(chsetup);
            
            obj.Hardware = hardware;
            obj.AcqSetup = acqsetup;
            obj.Trigger = trigger;
            obj.Average = average;
            obj.LevelMeter = levmeter;
            
            obj.FileSelect = CFileSelect({'mat'});
            
            obj.ThirdData = CGraphData();
            obj.ThirdTransferData = CGraphData();
            obj.RespData = CGraphData();
            obj.CohData = CGraphData();
            
            obj.InitFigure();
        end
        
        function SaveCallback(obj, ~)
            Result = obj.MeasurementResult; %#ok<NASGU>
            fname = fullfile(obj.FileSelect.getFullFileName());
            save(fname, 'Result');
        end
        
        function ImpactCallback(obj, hButton)
            
            % Check whether pretrig time is smaller than acquisition time
            if (obj.AcqSetup.T < obj.Trigger.preTrigTime)
                errordlg(CBabel.getInstance().getText('IMPACT_LONG_PRETRIGGER_ERROR'), ...
                    CBabel.getInstance().getText('IMPACT_ERROR'),...
                    'modal');
                return;
            end
            
            Fs = obj.AcqSetup.Fs;
            nSamples = obj.AcqSetup.nSamples;
            nAverage = obj.Average.nAverage;
            
            tScale = (0 : nSamples-1)' / Fs;
            fScale = (0:nSamples-1)'/nSamples*Fs; % frequency scale
            index = fScale <= Fs/2; % limit fscale to fs/2
            fScale = fScale(index);
            
            % Block user interfaces
            set(gcbf, 'UserData', get(gcbf, 'CloseRequestFcn'));
            set(gcbf, 'CloseRequestFcn', '');
            obj.hParentGroup.block();
            obj.AcqSetupView.block();
            obj.AverageView.block();

            set(hButton, 'Enable', 'off');
            
			% reset averaging counters
            obj.Average.reset();
            
			% get reference channel index for averaging
            refind = obj.ChannelSetup.getActiveReferenceIndex();
			% number of actual channels
            nCh = obj.ChannelSetup.getActiveChNum();
            
            nPretrigSamples = round(Fs * obj.Trigger.preTrigTime);
            trigLevel = obj.Trigger.trigLevel;
            
            try
                y = zeros(nSamples, nAverage, nCh);
                iAver = 0;
                while iAver < nAverage
                    obj.Trigger.initialize();
                    isTriggered = false;
                    trigCh = refind;
                    status = obj.Hardware.ImpactArm(nSamples, nPretrigSamples, obj.AcqSetup.Fs);

                    obj.Trigger.arm();
                    drawnow();
					
                    while ~isTriggered
                        [~, isTriggered] = obj.Hardware.ImpactWaitForTrigger(trigLevel, trigCh);
                    end
					
                    obj.Trigger.measure();
                    drawnow();
					
                    [~, yy]= obj.Hardware.ImpactAcq();
                    obj.Trigger.reset();
                    
                    obj.LevelMeter.UpdateLevels(yy);
                    
                    maxy = max(abs(yy), [], 1);
                    ovld_idx = maxy > obj.ChannelSetup.getUsedChannels().getRange();
                    if any(ovld_idx)
                        hDlg = errordlg('Overload', 'Impact overload', 'modal');
                        uiwait(hDlg);
                        continue;
                    end
                    
                    iAver = iAver + 1;
                    
                    % take channel sensitiviy and acceleration into account
                    y(:,iAver,:) = obj.ChannelSetup.voltage2eu(yy);

					% compute 3rd octave responses
					[Center, Y3] = time2third(Fs, y(:,1:iAver,:));
					Y3 = sqrt(mean(Y3.^2, 2));
					Y3 = reshape(Y3, size(Y3,1), size(Y3,3));
                    H3 = bsxfun(@times, Y3, 1./Y3(:,trigCh));

					% update pictures
					obj.ThirdData.UpdateData(Center, Y3);
                    obj.ThirdTransferData.UpdateData(Center, H3);
                    obj.RespData.UpdateData(tScale, squeeze(y(:,iAver,:)));
                    
                    drawnow();
                end % loop over averages
                
                y = obj.ChannelSetup.decompensate9234(y, obj.AcqSetup.Fs);
                
                x = y(:,:,refind);
                [~, Coh, ~] = transferest(x, y, 'noise');
                
				% limit to f < fs values
                Coh = Coh(index,:);
                
                obj.CohData.UpdateData(fScale, abs(Coh));
                
                obj.MeasurementResult.AcqSetup = obj.AcqSetup.dcopy();
                obj.MeasurementResult.Average = obj.Average.dcopy();
                obj.MeasurementResult.Channels = obj.ChannelSetup.getUsedChannels();
                obj.MeasurementResult.ExcitationSignal = x;
                obj.MeasurementResult.ResponseSignal = y;
                obj.MeasurementResult.ThirdOctaveFrequencies = Center;
                obj.MeasurementResult.ThirdOctaveSpectrum = Y3;
                obj.MeasurementResult.Coherence = Coh;
            catch ME
                errordlg(sprintf('%s\n%s\n%s:%d', ME.identifier, ME.message, ME.stack(1).name, ME.stack(1).line), '!!! Error !!!', 'modal');
            end
            
            % Unblock User Interfaces
            set(gcbf, 'CloseRequestFcn', get(gcbf, 'UserData'));
            obj.hParentGroup.unblock();
            obj.AcqSetupView.unblock();
            obj.AverageView.unblock();
            
            set(hButton, 'Enable', 'on');
        end
    end
    
    methods (Access = private)
        function InitFigure(obj)
            obj.AcqSetupView = CAcqSetupView(obj.hContainer, obj.AcqSetup);
            obj.AverageView = CAverageView(obj.hContainer, obj.Average);
            obj.TriggerView = CTriggerView(obj.hContainer, obj.Trigger);
            h2 = CLevelMeterView(obj.hContainer, obj.LevelMeter, 100);
            obj.FileSelectView = CFileSelectView(obj.hContainer, obj.FileSelect);
            butAcq = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton',...
                'Tag', 'START_TRANSFER', ...
                'String', CBabel.getInstance().getText('START_IMPACT'), ...
                'Callback', @(src, event)ImpactCallback(obj, src));
            butSav = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton',...
                'Tag', 'SAVE_TRANSFER',...
                'String', CBabel.getInstance().getText('SAVE_TRANSFER'), ...
                'Callback', @(src, event)SaveCallback(obj, src));
            h3 = CHardwareView(obj.hContainer, obj.Hardware);
            obj.ThirdDataView = CGraphDataView(obj.hContainer, obj.ThirdData, 'THIRD_OCTAVE_BAND_SPECTRUM');
            obj.ThirdTransferDataView = CGraphDataView(obj.hContainer, obj.ThirdTransferData, 'THIRD_OCTAVE_BAND_TRANSFER');
            obj.RespDataView = CGraphDataView(obj.hContainer, obj.RespData, 'RESPONSE_TIME_HISTORY');
            obj.CohDataView = CGraphDataView(obj.hContainer, obj.CohData, 'COHERENCE_FUNCTION');
            w = get(obj.AcqSetupView.hPanel, 'position');
            w = w(3);
            lMenu = {{obj.AcqSetupView.hPanel, [-1 0]};
                {obj.TriggerView.hPanel, [-1 0]};
                {obj.AverageView.hPanel, [-1 0]};
                {obj.FileSelectView.hPanel, [-1 0]};
                {butSav, [-1 CSetupClass.itemHeight]};
                {butAcq, [-1 CSetupClass.itemHeight]};
                {h2.hPanel, [-1 -1]};
                {h3.hPanel, [-1 0]}};
            content = {{{
			{obj.ThirdTransferDataView.hPanel, [-1 -1]}
			{obj.RespDataView.hPanel, [-1 -1]}
			}, [-1 -1]} {{
			{obj.ThirdDataView.hPanel, [-1 -1]}
			{obj.CohDataView.hPanel, [-1 -1]}
			}, [-1 -1]}};
            Layout = {{lMenu, [w, -1]} {content, [-1 -1]}};
            GuiLayout.ApplyLayout(obj.hContainer, Layout);
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
        end
    end
    
    methods
        function show(obj)
            %SHOW Called when the user opens this GUI module

            % warn if no reference channel is selected
            if isempty(obj.ChannelSetup.getActiveReferenceIndex)
                warndlg('Impact Testing without reference channel?', 'Impact testing warning', 'modal');
                return;
            end
            
            obj.LevelMeter.Reset();
            
            if obj.chSetupChanged
                Chan = obj.ChannelSetup.getUsedChannels();
                refind = obj.ChannelSetup.getActiveReferenceIndex();
                Col = {Chan.Color};
                Col = cell2mat(Col(:));

                obj.ThirdData.UpdateChannels(...
					cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    {CBabel.getInstance().getText('THIRD_OCTAVE_BAND_FREQUENCY')},...
					CQuantity.Frequency.EU,...
                    Chan.getQuantText(),...
					Chan.getEUText(),...
                    Col);
                obj.ThirdDataView.UpdateChannels();
                
                obj.ThirdTransferData.UpdateChannels(...
					cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    {CBabel.getInstance().getText('THIRD_OCTAVE_BAND_FREQUENCY')},...
                    CQuantity.Frequency.EU,...
                    Chan.getQuantText(),...
                    Chan.getEUText(),...
                    Col);
                obj.ThirdTransferDataView.UpdateChannels();
                
                obj.RespData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                obj.RespDataView.UpdateChannels();
                
                obj.CohData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Frequency.textID(), CQuantity.Frequency.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                
                obj.chSetupChanged = false;
            end
            
            obj.CohDataView.UpdateChannels();
            
            obj.ThirdTransferDataView.unblock();
            obj.RespDataView.unblock();
            obj.ThirdDataView.unblock();
            obj.CohDataView.unblock();
            
            status = obj.Hardware.ImpactInit(obj.ChannelSetup.InputChannels);
        end
        
        function hide(obj)
            %HIDE Called when the user switches to an other GUI module
            obj.ThirdTransferDataView.block();
            obj.RespDataView.block();
            obj.ThirdDataView.block();
            obj.CohDataView.block();
            
            status = obj.Hardware.ImpactClose();
        end
    end
end
