function HW = FindHardware(choices)
%FINDHARDWARE  Search for connected Hardwares
% HW = FINDHARDWARE(choices) searches for hardwares that can be used with
% the NiHu.Lab toolbox. Argument CHOICES is a cell array of strings with
% any of the following values:
%  'sim' for simulated input
%  'daq' for NI USB DAQ
%  'crio' for NI CompactRIO
%  'playrec' for playrec sound card
%  'daqmx' for NI daqmx network devices
% the default is {'sim', 'daq', 'crio', 'playrec', 'daqmx'}

if nargin < 1
    choices = {};
end

if isempty(choices) % automatic detection of choices
    switch sysnbit()
        case 32
            choices = {'sim', 'daq', 'crio', 'playrec', 'daqmx'};
        case 64
            choices = {'sim', 'daq', 'playrec', 'daqmx'};
    end
end

MainFigure = figure(...
    'Name', 'Select Hardware',...
    'NumberTitle', 'off', ...
    'Units', 'Pixels',...
    'MenuBar', 'None',...
    'Resize', 'off', ...
    'Color', get(0,'defaultUicontrolBackgroundColor'),...
    'Visible', 'on',...
    'CloseRequestFcn', '');

dataDir = fullfile(fileparts(mfilename('fullpath')), 'data');

Data = {
    'Simulator', 'matlab.jpg', 'sim', 'data.Sim = CSimulator;'
    'NI DAQ USB', 'DAQ.jpg', 'daq', 'data.DAQ = CDAQmxUSB;'
    'NI CRIO', 'cRIO.jpg', 'crio', 'data.FPGA = CFPGA;'
    'Playrec', 'Playrec.jpg', 'playrec', 'data.Playrec = CPlayrec;'
    'NI DAQmx', 'Playrec.jpg', 'daqmx', 'data.DAQmx = CDAQmx;'
    };


but = zeros(size(Data,1),1);
txt = zeros(size(Data,1),1);
lUp = cell(1,size(Data,1));
for i = 1 : size(Data,1)
    d = Data(i,:);
    im = double(imread(fullfile(dataDir, d{2})))/256;
    but(i) = uicontrol(...
        'parent', MainFigure,...
        'Style', 'PushButton',...
        'Callback', @(src, event)butCallback(src, d{3}), ...
        'TooltipString', d{1}, ...
        'CDATA', im, ...
        'enable', 'off', ...
        'units', 'pixels', 'position', [0 0 size(im,2) 128]);
    txt(i) = uicontrol(...
        'parent', MainFigure,...
        'Style', 'PushButton', ...
        'Callback', @(src, event)butCallback(src, d{3}), ...
        'units', 'pixels',...
        'position', [0 0 size(im,2) 40], ...
        'enable', 'off', ...
        'String', '');
    l = {{but(i), [0 0]};{txt(i), [0 0]}};
    lUp{i} = {l, [0 0]};
end

tInfo = uicontrol('parent', MainFigure, 'Style', 'Text', ...
    'BackgroundColor', [1 .8 .8], ...
    'units', 'pixels', 'position', [0 0 100 20], ...
    'String', '');

Layout = {
    {lUp, [0 0]}
    {tInfo, [-1 0]}
    };
GuiLayout.ApplyLayout(MainFigure, Layout);

movegui(MainFigure, 'center');
movegui(MainFigure, 'onscreen');
set(MainFigure, 'Visible', 'on');

set(tInfo, 'String', 'Discovering Hardware ...'); drawnow;

for i = 1 : size(Data,1)
    d = Data(i,:);
    if ismember(d{3}, choices)
        set(txt(i), 'String', sprintf('Connecting to %s ...', d{1})); drawnow;
        try
            eval(d{4});
            set(but(i), 'Enable', 'on');
            set(txt(i), 'Enable', 'on',...
                'String', 'Success');
            guidata(MainFigure, data);
        catch ME
            set(txt(i), 'Enable', 'on',...
                'String', 'Failure',...
                'Callback', @(src, event)errorDisplay(ME));
        end
        drawnow;
    end
end

set(tInfo, 'BackGroundColor', [.8 1 .8], ...
    'String', 'Please select your hardware now');
drawnow;

if exist('data', 'var')
    guidata(MainFigure, data);
end

set(MainFigure, 'CloseRequestFcn', 'closereq');

uiwait;

if ishandle(MainFigure)
    data = guidata(MainFigure);
    HW = data.HW;
    delete(MainFigure);
else
    error('NiHuLab:FindHardware:User Abort');
end
end

function butCallback(src, opt)
data = guidata(src);
switch lower(opt)
    case 'sim'
        data.HW = data.Sim;
    case 'daq'
        data.HW = data.DAQ;
    case 'crio'
        data.HW = data.FPGA;
    case 'playrec'
        data.HW = data.Playrec;
    case 'daqmx'
        data.HW = data.DAQmx;
end
guidata(src, data);
uiresume;
end

function errorDisplay(ME)
disp(ME);
end
