function w = colornoise(fs, nsamples, beta, type, fc)
%COLORNOISE Colored noise signal
%   W = COLORNOISE(FS, N, BETA, TYPE, FC) generates a colored noise signal
%   of N samples with sampling frequency FS. The the spectral energy
%   density of the colored noise obeys the f^BETA law. (BETA = 0
%   corresponds to white noise, BETA = -1 to pink noise, BETA = -2 to red
%   noise etc.). The parameter TYPE can be 'uniform' or 'gaussian'.
%   The parameter FC defines the cutoff frequency of the low pass filter
%   for the case of negative BETA paramteters.
%   W = COLORNOISE(FS, N, BETA, TYPE) operates with FC = 100 Hz.
%   W = COLORNOISE(FS, N, BETA) generates uniform colored noise with 100 Hz
%     cutoff..
%   W = COLORNOISE(FS, N) generates uniform white noise.
%
% Example
%   w = colornoise(44100, 10000, -1, 'uniform', 100);
%   soundsc(w, 44100);
%
% See also MLSSIGNAL SWEEPSIGNAL

% Copyright 2010-2010 Peter Fiala

% default parameters
if nargin < 5
    fc = 100;
end
if nargin < 4
    type = 'uniform';
end
if nargin < 3
    beta = 0;
end

% generate white noise
switch lower(type)
    case 'uniform'
        w = rand(nsamples,1)-.5;
    case 'gaussian'
        w = randn(nsamples,1);
    otherwise
        error('NiHuLab:Param', 'Unknown noise distribution ''%s''', type);
end

if beta ~= 0
    % generate filter coefficients
    f = abs((0:nsamples-1)'/nsamples*fs-fs/2);
    if beta < 0
        h = (f/fc).^(beta/2);
        h(f < fc) = 1;
    else
        h = f.^(beta/2);
    end
    w = ifft(fft(w, [], 1) .* fftshift(h,1), [], 1, 'symmetric');
end

% normalize noise
w = w - mean(w);
w = w / std(w);
end