function [H, C, h] = transferest(x, y, type, Fr)
%TRANSFEREST Estimate transfer of linear system
%   [FRF, C, IR] = TRANSFEREST(X, Y, TYPE, FR) estimates the frequency
%   response function, coherence and impulse response of a linear system
%   from its input X and output(s) Y. The transfer function is comptued along
%   the first dimension, and the averaging is performed along the second
%   dimension of Y. The transfer can be computed for arrays of arbitrary
%   number of dimensions, in this case, the transfer function is computed
%   separately for each dimension above 2.
%
% [FRF, C, IR] = TRANSFER(X, Y, 'noise') computes the transfer function
%    with stochastic input using a H1 estimate. In this case, size(X,2) =
%    size(Y,2) must be satisfied.
%
% [FRF, C, IR] = TRANSFER(X, Y, 'mls') computes the transfer function
%    with deterministic input using circular cross correlation, and
%    Coherence with a H1 estimate. In this case, size(X) = [N, 1] must be
%    satisfied.
%
% [FRF, C, IR] = TRANSFER(X, Y, 'sweep') computes the transfer function
%    with deterministic input using circular deconvolution, and a H1
%    estimate. In this case, size(X) = [N, 1] must be satisfied.
%
% FRF = TRANSFER(X, Y, 'harmonic', F/Fs) computes the transfer
%    coefficient with deterministic input using a vector voltage probe. In
%    this case, size(X) = [N, 1] must be satisfied.
%
% Examples:
%
% nSamples = 101;
% nAver = 12;
% n = 2;
% nCh = 25;
%
% x = rand(nSamples, nAver);
% y = rand(nSamples, nAver, n, nCh);
% [frf, c, ir] = transferest(x, y, 'noise');
%
% x = rand(nSamples, 1);
% y = rand(nSamples, nAver, n, nCh);
% [frf, c, ir] = transferest(x, y, 'sweep');
%
% x = rand(nSamples, 1);
% y = rand(nSamples, nAver, n, nCh);
% [frf, c, ir] = transferest(x, y, 'mls');
%
% x = rand(nSamples, 1);
% y = rand(nSamples, nAver, n, nCh);
% [frf, c] = transferest(x, y, 'harmonic', 100/1024);
%
% See also: circorr

% Copyright 2011 Peter Fiala


s = size(y);
nSamples = s(1);
nAverage = s(2);
nCh = prod(s(3:end));
y = reshape(y, [nSamples nAverage nCh]);

sx = size(x);
if length(sx) == length(s)-1
    if all (sx == s(1:end-1))
        x = reshape(x, [nSamples nAverage prod(sx(3:end))]);
    end
end

% Compute transfer with different methods for different excitations
switch lower(type)
    case 'harmonic'
        Fs = 48000;
        X = LevelPhase(x, Fs, 0, Fr*Fs);  % complex amplitude of excitation(s)
        Y = LevelPhase(y, Fs, 0, Fr*Fs);  % complex amplitude of responses
        if isscalar(X) % deterministic excitation
            H = mean(Y, 1) / X; % complex transfer
            C = abs(sum(Y, 1)).^2 ./ (nAverage * sum(abs(Y).^2, 1));
        else
            YX = mean(Y .* repmat(conj(X), [1, nCh]), 1);
            XX = mean(abs(X).^2, 1);
            YY = mean(abs(Y).^2, 1);
            H = YX ./ XX; % complex transfer
            C = abs(YX).^2 ./ (YY .* XX);
        end
        h = NaN;
    case 'mls'
        Y = fft(y, nSamples, 1);
        C = abs(squeeze(sum(Y,2))).^2 ./ squeeze(sum(abs(Y).^2, 2))/nAverage;
        y = squeeze(mean(y,2));
        h = circorr(y, x) / max(x)^2;
        H = fft(h, [], 1);
    case 'sweep'
        X = fft(x, [], 1);
        Y = fft(y, [], 1);
        C = abs(squeeze(sum(Y,2))).^2 ./ squeeze(sum(abs(Y).^2, 2))/nAverage;
        H = squeeze(mean(Y,2)) ./ repmat(X, 1, nCh);
        h = ifft(H, [], 1); % IR with inverse Fourier transform
    case 'noise'
        % Compute Fourier transforms
        X = fft(x, [], 1);
        Y = fft(y, [], 1);
        % Compute average cross spectrum and auto spectra
        n = nCh / size(x,3);
        YX = squeeze(mean(Y .* repmat(conj(X), [1, 1, n]), 2));
        XX = repmat(squeeze(mean(abs(X).^2, 2)), 1, n);
        YY = squeeze(mean(abs(Y).^2, 2));
        H = YX ./ XX; % FRF from auto and cross spectra
        h = ifft(H, [], 1); % IR with inverse Transform
        C = abs(YX).^2 ./ (YY .* XX); % Cohrence from cross spectra
end

sres = s(setdiff(1:end, 2));
if isscalar(sres)
	sres = [sres 1];
end
if ~strcmp(type, 'harmonic')
    H = reshape(H, sres);
    h = reshape(h, sres);
    C = reshape(C, sres);
else
    if length(size(sres)) > 2
        sres = sres(2:end);
        H = reshape(H, sres(2:end));
        C = reshape(C, sres(2:end));
    end
end

end